#include <stdio.h>
#include <stdlib.h>

#include "tree.h"

#define ANSI_COLOR_GREEN "\x1b[32m"
#define ANSI_COLOR_RED "\x1b[31m"
#define ANSI_COLOR_RESET "\x1b[0m"

/**
 * @brief Assert a condition
 * @details If the condition is false, print an error message
 *          and exit the program
 *
 * @param condition The condition to assert
 * @param message The message to print if the condition is false
 * @return 0 if the condition is true
 */
int assert(bool condition, const char *message) {
  if (!condition) {
    printf(ANSI_COLOR_RED "Assertion failed: %s\n" ANSI_COLOR_RESET, message);
    exit(1);
  } else {
    printf(ANSI_COLOR_GREEN "Assertion passed: %s\n" ANSI_COLOR_RESET, message);
  }
  return 0;
}

int main(void) {
  Tree tree = {NULL};

  // Some test insertions
  tree_insert(&tree, 1);
  tree_insert(&tree, 2);
  tree_insert(&tree, 3);
  tree_inorder(tree.root);
  printf("\n");

  // Basic sanity checks
  assert(tree.root->data == 1, "Root should be 1");
  assert(tree.root->right->data == 2, "Right should be 2");
  assert(tree.root->right->right->data == 3, "Right right should be 3");
  assert(tree_size(tree.root) == 3, "Tree size should be 3");

  // Test removal
  assert(tree_contains(tree.root, 2), "Tree should contain 2");
  tree_remove(&tree.root, 2);
  assert(!tree_contains(tree.root, 2), "Tree should not contain 2");

  // Clear the tree
  tree_clear(&tree);

  // Check if the tree is empty
  assert(tree.root == NULL, "Tree should be empty");
  assert(tree_size(tree.root) == 0, "Tree size should be 0");

  // Print size and inorder traversal
  printf("Tree size: %d\n", tree_size(tree.root));
  tree_inorder(tree.root);

  return 0;
}
